var fs   = require ('fs');
var os   = require ('os');
var path = require ('path');

var yauzl  = require ('yauzl');
var crypto = require ('crypto');

function createTempFolder (cb) {
	var fileName = path.join (os.tmpdir (), 'nomination.'+crypto.randomBytes(4).readUInt32LE(0));

	fs.mkdir (fileName, function (err) {
		// TODO: make something on error
		if (err) {
			cb (err)
			return;
		}
		cb (null, fileName);
	});
}

function mkdirParent (dirPath, mode, callback) {
	//Call the standard fs.mkdir
	if (!callback) {
		callback = mode;
		mode = undefined;
	}
	fs.mkdir(dirPath, mode, function(error) {
		//When it fail in this way, do the custom steps
		if (error && error.code === 'ENOENT') {
			//Create all the parents recursively
			mkdirParent (path.dirname (dirPath), mode, function (err) {
				//And then the directory
				mkdirParent (dirPath, mode, callback);
			});
			return;
		}
		//Manually run the callback since we used our own callback to do all these
		callback && callback(error);
	});
};

// file permissions, octal 0... isn't allowed in ES5, 0o... supported only in ES6
// 0644.toString(16)
var fileReadable = 0x1a4;
// 0755.toString(16)
var dirReadable  = 0x1ed;



function unzip (fileName, targetFolder, options) {

	if (!options) {
		options = {
			cb: function (err, data) {console.log (err, data);}
		};
	}

	var cb = options.cb;

	var archiveFiles = [];

	var err;

	var waitFileWrite = false;

	yauzl.open (fileName, function(error, zipfile) {
		if (error) {
			cb (error, archiveFiles);
			return;
		}
		var remains = 0;

		function fileOpDone (error) {
			err = error;
			remains --;
			if (!remains && waitFileWrite) {
				cb (err, archiveFiles);
			}
		}

		zipfile.on ("entry", function(entry) {
			console.log ('zip file entry', entry);

			if (!options.macosx && /^__MACOSX\//.test(entry.fileName)) {
				// TODO: mac os resource forks and extended attrs
				// fs-xattr, node-xattr-file
				// http://apple.stackexchange.com/questions/141861/do-zip-files-preserve-all-hfs-features-when-created-using-finders-compress-co
				return;
			}

			if ((!options.macosx || os.platform() !== "darwin") && /(?:^|\/)\.(?:_\.|DS_Store)/.test(entry.fileName)) {
				// files like ._filename and .DS_Store is a mac only things
				return;
			}

			// http://unix.stackexchange.com/questions/14705/the-zip-formats-external-file-attribute
			// from stat(2)
			var modeHi = (entry.externalFileAttributes >> 16) & 0xffff;
			var isDir     = 0x4 & modeHi; // #define S_IFDIR  0040000  /* directory */
			var isFile    = 0x8 & modeHi; // #define S_IFREG  0100000  /* regular */
			var isSymlink = 0xa & modeHi; // #define S_IFLNK  0120000  /* symbolic link */

			var iUmask = ~process.umask ();
			var perms = iUmask & (entry.externalFileAttributes || fileReadable);
			if (isDir) {
				// TODO: not used
				perms = iUmask & (entry.externalFileAttributes || dirReadable);
			}

			if (/\/$/.test(entry.fileName)) {
				// directory file names end with '/'
				// archiveFolders.push (entry.fileName);
				return;
			}

			archiveFiles.push (entry);
			remains++;
			zipfile.openReadStream (entry, function(error, readStream) {
				if (error) {
					fileOpDone (error);
					return;
				}

				// console.log (targetFolder, entry.fileName);

				var entryFile   = path.join (targetFolder, entry.fileName);
				if (options.replacePath) {
					entryFile = entryFile.replace (options.replacePath[0], options.replacePath[1])
				}
				var entryFolder = path.dirname (entryFile);

				if (entryFolder) {
					mkdirParent (
						entryFolder,
						isSymlink ? writeSymlink : writeFile
					);
				} else {
					isSymlink ? writeSymlink () : writeFile ();
				}

				function writeFile () {
					var writeStream = fs.createWriteStream (entryFile, {mode: perms});
					writeStream.on ('error', fileOpDone);
					writeStream.on ('finish', fileOpDone);
					readStream.pipe (writeStream);
				}

				function writeSymlink () {
					var buf = new Buffer(0);
					readStream.on ('data', function (d) { buf += d; });
					readStream.on ('end', function () {
						var linkSrc = buf.toString ('utf8');
						fs.symlink (linkSrc, entryFile, fileOpDone);
					});
				}
			});
		});
		// archive read complete
		zipfile.on ('end', function () {
			// console.log ('zip file end', remains);
			if (remains) {
				waitFileWrite = true;
			} else {
				cb (err, archiveFiles);
			}
		});
	});
}

if (typeof describe !== "undefined") {
	return;
}

if (process.argv[2]) {
	createTempFolder (function (err, folderName) {
		if (err) {
			console.error (err);
			return;
		}

		unzip (process.argv[2], folderName, {dryRun: process.argv[3], cb: function (err, archiveFiles) {
			console.log (err);
			console.log (archiveFiles);
		}});
	});
}
